@@ -1,5 +1,7 @@
 #############################################################
-# Copyright (c) 2005-2023 VMware, Inc.
+# Copyright (c) 2005-2025 Broadcom. All Rights Reserved.
+# The term "Broadcom" refers to Broadcom Inc.
+# and/or its subsidiaries.
 #############################################################
 
 # @file connect.py
@@ -18,9 +20,7 @@ import ssl
 from sys import exc_info
 from xml.parsers.expat import ExpatError
 
-from six.moves import http_client, urllib
-from six import reraise
-
+from pyVmomi.five import PY3, HTTPConnection, HTTPSConnection
 from pyVmomi import SessionOrientedStub, SoapStubAdapter, vim, vmodl
 from pyVmomi.SoapAdapter import CONNECTION_POOL_IDLE_TIMEOUT_SEC
 from pyVmomi.VmomiSupport import (GetServiceVersions, IsChildVersion, nsMap,
@@ -132,8 +132,8 @@ class VimSessionOrientedStub(SessionOrientedStub):
 
         def _doLogin(soapStub):
             from . import sso
-            cert = soapStub.schemeArgs['cert_file']
-            key = soapStub.schemeArgs['key_file']
+            cert = soapStub.certFile
+            key = soapStub.certKeyFile
             authenticator = sso.SsoAuthenticator(sts_url=stsUrl,
                                                  sts_cert=stsCert)
 
@@ -177,8 +177,8 @@ class VimSessionOrientedStub(SessionOrientedStub):
 
         def _doLogin(soapStub):
             from . import sso
-            cert = soapStub.schemeArgs['cert_file']
-            key = soapStub.schemeArgs['key_file']
+            cert = soapStub.certFile
+            key = soapStub.certKeyFile
             authenticator = sso.SsoAuthenticator(sts_url=stsUrl,
                                                  sts_cert=stsCert)
             samlAssertion = authenticator.get_bearer_saml_assertion(
@@ -209,6 +209,7 @@ def Connect(host='localhost',
             httpProxyHost=None,
             httpProxyPort=80,
             thumbprint=None,
+            serverPemCert=None,
             sslContext=None,
             httpConnectionTimeout=None,
             connectionPoolTimeout=CONNECTION_POOL_IDLE_TIMEOUT_SEC,
@@ -216,10 +217,11 @@ def Connect(host='localhost',
             tokenType=None,
             disableSslCertValidation=False,
             customHeaders=None,
+            sessionId=None,
             # Deprecated
             b64token=None,
             # Deprecated
-            mechanism='userpass'):
+            mechanism=None):
     """
     Connect to the specified server, login and return the service
     instance object.
@@ -257,8 +259,17 @@ def Connect(host='localhost',
     @type  httpProxyHost: string
     @param httpProxyPort The proxy server port.
     @type  httpProxyPort: int
-    @param thumbprint: host cert thumbprint
+    @param thumbprint: **** Deprecated. Use serverPemCert instead.
+                        If both fields are set, thumbprint should match
+                        serverPemCert.
+                        The SHA1/SHA256/SHA512 thumbprint of the server's
+                        SSL certificate.
+                        Some use a thumbprint of the form xx:xx:xx..:xx.
+                        We ignore the ":" characters.
     @type  thumbprint: string
+    @param serverPemCert: PEM-encoded SSL certificate of the
+                            host to which we are connecting.
+    @type  serverPemCert: string
     @param sslContext: SSL Context describing the various SSL options. It is only
                         supported in Python 2.7.9 or higher.
     @type  sslContext: SSL.Context
@@ -276,6 +287,9 @@ def Connect(host='localhost',
     @param disableSslCertValidation: Creates an unverified SSL context when True.
     @type  customHeaders: dict
     @param customHeaders: Dictionary with custom HTTP headers.
+    @type  sessionId: string
+    @param sessionId: Allows usage of an existing session.
+                        If supplied Login will not be attempted.
     @param b64token: base64 encoded token
            *** Deprecated: Use token instead ***
     @type  b64token: string
@@ -292,18 +306,12 @@ def Connect(host='localhost',
     elif not version:
         version = "vim.version.version9"
 
-    # Backwards compatibility for the deprecated arguments -
-    # b64token and mechanism
-    msg = "The default connection type uses credentials. " \
-          "If you want to authenticate with a token, set 'token' and 'tokenType'."
-    if mechanism == "sspi":
-        if b64token:
-            token = b64token
-            tokenType = TOKEN_TYPE_SSPI
-        else:
-            raise Exception("The b64token parameter is mandatory. " + msg)
-    elif mechanism != "userpass":
-        raise Exception("Not supported mechanism. " + msg)
+    if b64token or mechanism:
+        raise Exception(
+            "'b64token' and 'mechanism' are no longer supported. "
+            "The default connection type uses credentials. "
+            "If you want to authenticate with a token,"
+            "set 'token' and 'tokenType'.")
 
     si, stub = __Login(host,
                        port,
@@ -318,12 +326,14 @@ def Connect(host='localhost',
                        httpProxyHost,
                        httpProxyPort,
                        thumbprint,
+                       serverPemCert,
                        sslContext,
                        httpConnectionTimeout,
                        connectionPoolTimeout,
                        token=token,
                        tokenType=tokenType,
-                       customHeaders=customHeaders)
+                       customHeaders=customHeaders,
+                       sessionId=sessionId)
     SetSi(si)
 
     return si
@@ -388,12 +398,14 @@ def __Login(host,
             httpProxyHost,
             httpProxyPort,
             thumbprint,
+            serverPemCert,
             sslContext,
             httpConnectionTimeout,
             connectionPoolTimeout,
             token,
             tokenType,
-            customHeaders):
+            customHeaders,
+            sessionId):
     """
     Private method that performs the actual Connect and returns a
     connected service instance object.
@@ -422,8 +434,17 @@ def __Login(host,
     @type  httpProxyHost: string
     @param httpProxyPort The proxy server port.
     @type  httpProxyPort: int
-    @param thumbprint: host cert thumbprint
+    @param thumbprint: **** Deprecated. Use serverPemCert instead.
+                        If both fields are set, thumbprint should match
+                        serverPemCert.
+                        The SHA1/SHA256/SHA512 thumbprint of the server's
+                        SSL certificate.
+                        Some use a thumbprint of the form xx:xx:xx..:xx.
+                        We ignore the ":" characters.
     @type  thumbprint: string
+    @param serverPemCert: PEM-encoded SSL certificate of the
+                            host to which we are connecting.
+    @type  serverPemCert: string
     @param sslContext: SSL Context describing the various SSL options. It is only
                         supported in Python 2.7.9 or higher.
     @type  sslContext: SSL.Context
@@ -439,6 +460,9 @@ def __Login(host,
     @param tokenType: Select which type of Authentication and Authorization token to use.
     @type  customHeaders: dict
     @param customHeaders: Dictionary with custom HTTP headers.
+    @type  sessionId: string
+    @param sessionId: Allows usage of an existing session.
+                        If supplied Login will not be attempted.
     """
 
     # XXX remove the adapter and service arguments once dependent code is fixed
@@ -468,11 +492,13 @@ def __Login(host,
         httpProxyHost=httpProxyHost,
         httpProxyPort=httpProxyPort,
         thumbprint=thumbprint,
+        serverPemCert=serverPemCert,
         sslContext=sslContext,
         httpConnectionTimeout=httpConnectionTimeout,
         connectionPoolTimeout=connectionPoolTimeout,
         samlToken=samlToken,
-        customHeaders=customHeaders)
+        customHeaders=customHeaders,
+        sessionId=sessionId)
 
     # Get Service instance
     si = vim.ServiceInstance("ServiceInstance", stub)
@@ -487,12 +513,15 @@ def __Login(host,
         # failures easier since the fault will also include where inside the
         # library the fault occurred. Without the traceback we have no idea
         # why the connection failed beyond the message string.
-        (type, value, traceback) = exc_info()
         fault = vim.fault.HostConnectFault(msg=str(e))
-        if traceback:
-            reraise(vim.fault.HostConnectFault, fault, traceback)
-        else:
-            raise fault
+        if PY3:
+            _, _, tb = exc_info()
+            if tb:
+                raise fault.with_traceback(tb)
+        raise fault
+
+    if sessionId:
+        return si, stub
 
     # Get a ticket if we're connecting to localhost and password is not specified
     if host == 'localhost' and not pwd and not token:
@@ -580,38 +609,6 @@ class SmartConnection(object):
             self.si = None
 
 
-# Deprecated, use __GetElementTree method instead
-def __GetElementTreeFromUrl(url, sslContext):
-    """
-    Private method that returns an ElementTree for the XML document referenced by
-    the url.
-
-    @param url: URL
-    @type  url: string
-    @param sslContext: SSL Context describing the various SSL options. It is only
-                        supported in Python 2.7.9 or higher.
-    @type  sslContext: SSL.Context
-    """
-
-    tree = ElementTree()
-
-    try:
-        urlopen_kwargs = {}
-        if sslContext:
-            # We need to pass the ssl context as kwargs because 'context' parameter
-            # was added in Python 2.7.9, so passing it as a normal parameter would
-            # fail in earlier versions. Please see
-            # https://www.python.org/dev/peps/pep-0476/ for more details.
-            urlopen_kwargs['context'] = sslContext
-        with closing(urllib.request.urlopen(url, **urlopen_kwargs)) as sock:
-            if sock.getcode() == 200:
-                tree.parse(sock)
-                return tree
-    except ExpatError:
-        pass
-    return None
-
-
 def __GetElementTree(protocol, server, port, path, sslContext,
                      httpProxyHost, httpProxyPort,
                      customHeaders):
@@ -641,13 +638,13 @@ def __GetElementTree(protocol, server, port, path, sslContext,
 
     if httpProxyHost:
         kwargs = {"context": sslContext} if sslContext else {}
-        conn = http_client.HTTPSConnection(httpProxyHost, port=httpProxyPort, **kwargs)
+        conn = HTTPSConnection(httpProxyHost, port=httpProxyPort, **kwargs)
         conn.set_tunnel(server, port, headers)
     elif protocol == "https":
         kwargs = {"context": sslContext} if sslContext else {}
-        conn = http_client.HTTPSConnection(server, port=port, **kwargs)
+        conn = HTTPSConnection(server, port=port, **kwargs)
     elif protocol == "http":
-        conn = http_client.HTTPConnection(server, port=port)
+        conn = HTTPConnection(server, port=port)
     else:
         raise Exception("Protocol " + protocol + " not supported.")
     conn.request(method="GET", url=path, headers=headers)
@@ -799,6 +796,7 @@ def SmartStubAdapter(host='localhost',
                      httpProxyPort=80,
                      sslProxyPath=None,
                      thumbprint=None,
+                     serverPemCert=None,
                      cacertsFile=None,
                      preferredApiVersions=None,
                      acceptCompressedResponses=True,
@@ -807,7 +805,8 @@ def SmartStubAdapter(host='localhost',
                      httpConnectionTimeout=None,
                      connectionPoolTimeout=CONNECTION_POOL_IDLE_TIMEOUT_SEC,
                      disableSslCertValidation=False,
-                     customHeaders=None):
+                     customHeaders=None,
+                     sessionId=None):
     """
     Determine the most preferred API version supported by the specified server,
     then create a soap stub adapter using that version
@@ -825,6 +824,9 @@ def SmartStubAdapter(host='localhost',
     @param disableSslCertValidation: Creates an unverified SSL context when True.
     @type  customHeaders: dict
     @param customHeaders: Dictionary with custom HTTP headers.
+    @type  sessionId: string
+    @param sessionId: Allows usage of an existing session.
+                        If supplied Login will not be attempted.
     """
     if preferredApiVersions is None:
         preferredApiVersions = GetServiceVersions('vim25')
@@ -853,6 +855,7 @@ def SmartStubAdapter(host='localhost',
                            httpProxyPort=httpProxyPort,
                            sslProxyPath=sslProxyPath,
                            thumbprint=thumbprint,
+                           serverPemCert=serverPemCert,
                            cacertsFile=cacertsFile,
                            version=supportedVersion,
                            acceptCompressedResponses=acceptCompressedResponses,
@@ -860,7 +863,8 @@ def SmartStubAdapter(host='localhost',
                            sslContext=sslContext,
                            httpConnectionTimeout=httpConnectionTimeout,
                            connectionPoolTimeout=connectionPoolTimeout,
-                           customHeaders=customHeaders)
+                           customHeaders=customHeaders,
+                           sessionId=sessionId)
 
 
 def SmartConnect(protocol='https',
@@ -876,6 +880,7 @@ def SmartConnect(protocol='https',
                  httpProxyHost=None,
                  httpProxyPort=80,
                  thumbprint=None,
+                 serverPemCert=None,
                  sslContext=None,
                  httpConnectionTimeout=None,
                  connectionPoolTimeout=CONNECTION_POOL_IDLE_TIMEOUT_SEC,
@@ -883,10 +888,11 @@ def SmartConnect(protocol='https',
                  tokenType=None,
                  disableSslCertValidation=False,
                  customHeaders=None,
+                 sessionId=None,
                  # Deprecated
                  b64token=None,
                  # Deprecated
-                 mechanism='userpass'):
+                 mechanism=None):
     """
     Determine the most preferred API version supported by the specified server,
     then connect to the specified server using that API version, login and return
@@ -927,8 +933,17 @@ def SmartConnect(protocol='https',
     @type  httpProxyHost: string
     @param httpProxyPort The proxy server port.
     @type  httpProxyPort: int
-    @param thumbprint: host cert thumbprint
+    @param thumbprint: **** Deprecated. Use serverPemCert instead.
+                        If both fields are set, thumbprint should match
+                        serverPemCert.
+                        The SHA1/SHA256/SHA512 thumbprint of the server's
+                        SSL certificate.
+                        Some use a thumbprint of the form xx:xx:xx..:xx.
+                        We ignore the ":" characters.
     @type  thumbprint: string
+    @param serverPemCert: PEM-encoded SSL certificate of the
+                            host to which we are connecting.
+    @type  serverPemCert: string
     @param sslContext: SSL Context describing the various SSL options. It is only
                         supported in Python 2.7.9 or higher.
     @type  sslContext: SSL.Context
@@ -944,6 +959,9 @@ def SmartConnect(protocol='https',
     @param disableSslCertValidation: Creates an unverified SSL context when True.
     @type  customHeaders: dict
     @param customHeaders: Dictionary with custom HTTP headers.
+    @type  sessionId: string
+    @param sessionId: Allows usage of an existing session.
+                        If supplied Login will not be attempted.
     @param b64token: base64 encoded token
            *** Deprecated: Use token instead ***
     @type  b64token: string
@@ -981,6 +999,7 @@ def SmartConnect(protocol='https',
                    httpProxyHost=httpProxyHost,
                    httpProxyPort=httpProxyPort,
                    thumbprint=thumbprint,
+                   serverPemCert=serverPemCert,
                    sslContext=sslContext,
                    httpConnectionTimeout=httpConnectionTimeout,
                    connectionPoolTimeout=connectionPoolTimeout,
@@ -988,58 +1007,11 @@ def SmartConnect(protocol='https',
                    tokenType=tokenType,
                    disableSslCertValidation=disableSslCertValidation,
                    customHeaders=customHeaders,
+                   sessionId=sessionId,
                    b64token=b64token,
                    mechanism=mechanism)
 
 
-# Deprecated
-def OpenUrlWithBasicAuth(url, user='root', pwd='', verify=True):
-    """
-    Open the specified URL, using HTTP basic authentication to provide
-    the specified credentials to the server as part of the request.
-    Returns the response as a file-like object.
-    """
-    pwMgr = urllib.request.HTTPPasswordMgrWithDefaultRealm()
-    pwMgr.add_password(None, url, user, pwd)
-    authHandler = urllib.request.HTTPBasicAuthHandler(pwMgr)
-    if verify:
-        opener = urllib.request.build_opener(authHandler)
-    else:
-        sslContext = ssl._create_unverified_context()
-        sslHandler = urllib.request.HTTPSHandler(context=sslContext)
-        opener = urllib.request.build_opener(sslHandler, authHandler)
-    return opener.open(url)
-
-
-# Deprecated
-def OpenPathWithStub(path, stub, verify=True):
-    """
-    Open the specified path using HTTP, using the host/port/protocol
-    associated with the specified stub.  If the stub has a session cookie,
-    it is included with the HTTP request.  Returns the response as a
-    file-like object.
-    """
-    if not hasattr(stub, 'scheme'):
-        raise vmodl.fault.NotSupported()
-    elif stub.scheme == http_client.HTTPConnection:
-        protocol = 'http'
-        verify = False
-    elif stub.scheme == http_client.HTTPSConnection:
-        protocol = 'https'
-    else:
-        raise vmodl.fault.NotSupported()
-    hostPort = stub.host
-    url = '{0}://{1}{2}'.format(protocol, hostPort, path)
-    request = urllib.request.Request(url)
-    if stub.cookie:
-        request.add_header('Cookie', stub.cookie)
-    if verify:
-        return urllib.request.urlopen(request)
-    else:
-        sslContext = ssl._create_unverified_context()
-        return urllib.request.urlopen(request, context=sslContext)
-
-
 def IsManagedHost():
     """
     Check whether the host is managed by vCenter
