@@ -1,8 +1,10 @@
-# **********************************************************
-# Copyright (c) 2008-2022 VMware, Inc.
-# **********************************************************
+# Copyright (c) 2008-2024 Broadcom. All Rights Reserved.
+# The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
+
 from .VmomiSupport import CreateVersion, parentMap
 
+kind = "OSS"
+
 
 # Version-specific initialization
 def Init():
