@@ -1,36 +1,24 @@
-# Copyright (c) 2005-2023 VMware, Inc.
-"""VMOMI support code
+# Copyright (c) 2005-2025 Broadcom. All Rights Reserved.
+# The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 
-This module contains support for VMOMI abstractions such as VMODL types,
-Versions, etc.
-"""
+# Support for VMOMI abstractions such as VMODL types, versions, etc.
 
 import base64
 import threading
 from datetime import datetime
-from sys import version_info
+from functools import partial
 
-import six
-from six import PY3, binary_type, string_types
-from six.moves import map, range
+from .five import PY3, binary_type, str_type, text_type, iteritems, itervalues
+if PY3:
+    from functools import cmp_to_key
+    from sys import intern
+else:
+    from .five import map, range
 
 from . import Iso8601
-from . import _allowGetSet
-from . import _allowCapitalizedNames
-from . import _binaryIsBytearray
 
-if version_info[0] >= 3:
-    from functools import cmp_to_key
 NoneType = type(None)
 
-try:
-    from six.moves import intern
-except ImportError:
-    # The old version of six package (< 1.8.0) doesn't have intern mapping so
-    # we need to tolerate the import error here.
-    def intern(s):
-        return s
-
 (F_LINK, F_LINKABLE, F_OPTIONAL, F_SECRET) = [1 << x for x in range(4)]
 
 BASE_VERSION = 'vmodl.version.version0'
@@ -172,13 +160,13 @@ class LazyObject(Object):
                 raise AttributeError(attr)
 
 
-class Link(six.text_type):
+class Link(text_type):
     def __new__(cls, obj):
-        if isinstance(obj, string_types):
-            return six.text_type.__new__(cls, obj)
+        if isinstance(obj, str_type):
+            return text_type.__new__(cls, obj)
         elif isinstance(obj, DataObject):
             if obj.key:
-                return six.text_type.__new__(cls, obj.key)
+                return text_type.__new__(cls, obj.key)
             raise AttributeError("DataObject does not have a key to link")
         else:
             raise ValueError
@@ -398,19 +386,28 @@ class ManagedObject(object):
 
     _InvokeAccessor = staticmethod(_InvokeAccessor)
 
+    # In contrast with the Python convention for "private" methods,
+    # the ManagedObject and DataObject method names beginning with
+    # an underscore denote public, officially supported system methods.
+    # The execution of these methods does not involve a remote call.
+
     # Get the ID of a managed object
+    # Public non-remote method. Officially supported.
     def _GetMoId(self):
         return self._moId
 
     # Get the serverGuid of a managed object
+    # Public non-remote method. Officially supported.
     def _GetServerGuid(self):
         return self._serverGuid
 
     # Get the stub of a managed object
+    # Public non-remote method. Officially supported.
     def _GetStub(self):
         return self._stub
 
     # Get a list of all properties of this type and base types
+    # Public non-remote method. Officially supported.
     #
     # @param cls The managed object type
     @classmethod
@@ -430,6 +427,7 @@ class ManagedObject(object):
         return result
 
     # Get a list of all methods of this type and base types
+    # Public non-remote method. Officially supported.
     #
     # @param cls The managed object type
     @classmethod
@@ -447,6 +445,7 @@ class ManagedObject(object):
         return result
 
     # Lookup a method for a given managed object type
+    # Public non-remote method. Officially supported.
     #
     # @param type the type
     # @param name the name of the property
@@ -467,15 +466,13 @@ class ManagedObject(object):
 
     __delattr__ = __setattr__
 
-    if _allowGetSet:
-
-        def __getattr__(self, name):
-            if name.startswith("Get"):
-                return lambda: getattr(self, name[3].lower() + name[4:])
-            elif name.startswith("Set"):
-                return lambda val: setattr(self, name[3].lower() + name[4:],
-                                           val)
-            raise AttributeError(name)
+    def __getattr__(self, name):
+        if name.startswith("Get"):
+            return lambda: getattr(self, name[3].lower() + name[4:])
+        elif name.startswith("Set"):
+            return lambda val: setattr(self, name[3].lower() + name[4:],
+                                       val)
+        raise AttributeError(name)
 
     # The equality test of ManagedObject is for client side only and might
     # not be appropriate for server side objects. The server side object has
@@ -534,7 +531,13 @@ class DataObject(object):
         for (k, v) in list(kwargs.items()):
             setattr(self, k, v)
 
+    # In contrast with the Python convention for "private" methods,
+    # the ManagedObject and DataObject method names beginning with
+    # an underscore denote public, officially supported system methods.
+    # The execution of these methods does not involve a remote call.
+
     # Get a list of all properties of this type and base types
+    # Public non-remote method. Officially supported.
     #
     # @param cls the data object type
     @classmethod
@@ -695,6 +698,40 @@ def _CheckNestedClasses(typ, parent):
         return typ
 
 
+# A flag to control whether new type definitions will be frozen
+_freezeDefintions = False
+
+# Control whether new type definitions will be frozen. When True is passed,
+# subsequent calls to CreateDataType, CreateManagedType and CreateEnumType will
+# mark the new type definition as frozen. When False is passed, the usual
+# mode will be restored.
+def SetFreezeDefinitions(enabled):
+    global _freezeDefintions
+    _freezeDefintions = enabled
+
+# The set of frozen names, specified as (namespace, name) pairs. This set is
+# accessible only to the _TryFreezeName and _IsNameFrozen
+# functions.
+_frozenNames = set()
+
+# If the _freezeDefinitions flag is True, adds the type to the set of frozen
+# type definitions. Otherwise has no effect.
+def _TryFreezeName(typeNs, name, frozenNames):
+    if _freezeDefintions:
+        frozenNames.add((typeNs, intern(name)))
+
+_TryFreezeName = partial(_TryFreezeName, frozenNames = _frozenNames)
+
+# Checks whether the specified type has been frozen
+def _IsNameFrozen(typeNs, name, frozenNames):
+    return (typeNs, intern(name)) in frozenNames
+
+_IsNameFrozen = partial(_IsNameFrozen, frozenNames = _frozenNames)
+
+# Make the frozen types set inaccessible outside the two functions which use it
+del _frozenNames
+
+
 # Create and Load a data object type at once
 #
 # @param vmodlName the VMODL name of the type
@@ -717,17 +754,22 @@ def CreateAndLoadDataType(vmodlName, wsdlName, parent, version, props):
 # @param props properties of the type
 def CreateDataType(vmodlName, wsdlName, parent, version, props):
     with _lazyLock:
+        typeNs = GetInternedWsdlNamespace(version)
+        if _IsNameFrozen(typeNs, wsdlName):
+            return
+
         dic = [vmodlName, wsdlName, parent, version, props]
         names = vmodlName.split(".")
         if _allowCapitalizedNames:
             vmodlName = ".".join(name[0].lower() + name[1:] for name in names)
         _AddToDependencyMap(names)
-        typeNs = GetInternedWsdlNamespace(version)
 
         _dataDefMap[vmodlName] = dic
         _wsdlDefMap[(typeNs, wsdlName)] = dic
         _wsdlTypeMapNSs.add(typeNs)
 
+        _TryFreezeName(typeNs, wsdlName)
+
 
 # Load a data object type
 # This function also loads the parent of the type if it's not loaded yet
@@ -777,16 +819,15 @@ def LoadDataType(vmodlName, wsdlName, parent, version, props):
             "_version": version
         }
 
-        if _allowGetSet:
-            for property in propList:
-                name = property.name
-                suffix = name[0].upper() + name[1:]
-                tGlobals = {}
-                exec(
-                    'def Get%s(self): return getattr(self, "%s")' %
-                    (suffix, name) + '\n' +
-                    'def Set%s(self, v): setattr(self, "%s", v)' %
-                    (suffix, name), tGlobals, dic)
+        for property in propList:
+            name = property.name
+            suffix = name[0].upper() + name[1:]
+            tGlobals = {}
+            exec(
+                'def Get%s(self): return getattr(self, "%s")' %
+                (suffix, name) + '\n' +
+                'def Set%s(self, v): setattr(self, "%s", v)' %
+                (suffix, name), tGlobals, dic)
 
         vmodlParent = GetVmodlType(parent)
         result = _AddType(LazyType(vmodlName, (vmodlParent, ), dic))
@@ -838,13 +879,16 @@ def CreateAndLoadManagedType(vmodlName, wsdlName, parent, version, props,
 # @param methods methods of the type
 def CreateManagedType(vmodlName, wsdlName, parent, version, props, methods):
     with _lazyLock:
+        typeNs = GetInternedWsdlNamespace(version)
+        if _IsNameFrozen(typeNs, wsdlName):
+            return
+
         dic = [vmodlName, wsdlName, parent, version, props, methods]
         names = vmodlName.split(".")
         if _allowCapitalizedNames:
             vmodlName = ".".join(name[0].lower() + name[1:] for name in names)
 
         _AddToDependencyMap(names)
-        typeNs = GetInternedWsdlNamespace(version)
 
         if methods:
             for meth in methods:
@@ -859,6 +903,8 @@ def CreateManagedType(vmodlName, wsdlName, parent, version, props, methods):
         _wsdlDefMap[(typeNs, wsdlName)] = dic
         _wsdlTypeMapNSs.add(typeNs)
 
+        _TryFreezeName(typeNs, wsdlName)
+
 
 # Load a managed object type
 # This function also loads the parent of the type if it's not loaded yet
@@ -961,18 +1007,23 @@ def CreateAndLoadEnumType(vmodlName, wsdlName, version, values):
 # @param values enum values
 def CreateEnumType(vmodlName, wsdlName, version, values):
     with _lazyLock:
+        typeNs = GetInternedWsdlNamespace(version)
+        if _IsNameFrozen(typeNs, wsdlName):
+            return
+
         dic = [vmodlName, wsdlName, version, values]
         names = vmodlName.split(".")
         if _allowCapitalizedNames:
             vmodlName = ".".join(name[0].lower() + name[1:] for name in names)
 
         _AddToDependencyMap(names)
-        typeNs = GetInternedWsdlNamespace(version)
 
         _enumDefMap[vmodlName] = dic
         _wsdlDefMap[(typeNs, wsdlName)] = dic
         _wsdlTypeMapNSs.add(typeNs)
 
+        _TryFreezeName(typeNs, wsdlName)
+
 
 # Load an enum type
 #
@@ -1034,7 +1085,7 @@ def _areBasicTypes(info, valType):
             or issubclass(info, long) and (issubclass(valType, int)
                                            or issubclass(valType, long))
             or issubclass(info, float) and issubclass(valType, float)
-            or issubclass(info, string_types) and issubclass(valType, string_types)
+            or issubclass(info, str_type) and issubclass(valType, str_type)
             or issubclass(info, binary_type) and issubclass(valType, binary_type))
 
 
@@ -1077,7 +1128,7 @@ def CheckField(info, val):
                     if issubclass(valType, GetVmodlType(info.expectedType)):
                         return
                 elif issubclass(info.type.Item, Enum) and issubclass(
-                        valType.Item, string_types):
+                        valType.Item, str_type):
                     # Allow String array object to be assigned to enum array
                     return
             elif val:
@@ -1221,7 +1272,7 @@ def GetWsdlTypes():
     with _lazyLock:
         for ns, name in _wsdlDefMap:
             GetWsdlType(ns, name)
-        return six.itervalues(_wsdlTypeMap)
+        return itervalues(_wsdlTypeMap)
 
 
 # Get the qualified XML schema name (ns, name) of a type
@@ -1373,13 +1424,13 @@ def GetServiceVersions(namespace):
             return 1
         return (a > b) - (a < b)
 
-    if version_info[0] >= 3:
+    if PY3:
         return sorted(
-            [v for (v, n) in six.iteritems(serviceNsMap) if n == namespace],
+            [v for (v, n) in iteritems(serviceNsMap) if n == namespace],
             key=cmp_to_key(compare))
     else:
         return sorted(
-            [v for (v, n) in six.iteritems(serviceNsMap) if n == namespace],
+            [v for (v, n) in iteritems(serviceNsMap) if n == namespace],
             compare)
 
 
@@ -1484,7 +1535,7 @@ def GetCompatibleType(type, version):
 
 # Invert an injective mapping
 def InverseMap(map):
-    return dict([(v, k) for (k, v) in six.iteritems(map)])
+    return dict([(v, k) for (k, v) in iteritems(map)])
 
 
 def GetVmodlNs(version):
@@ -1532,16 +1583,18 @@ class _MaturitySet:
         """
         vmodlNs = GetVmodlNs(version)
 
-        # TODO fix the VSAN-related part of vcenter-all to enable the assert
-        # assert not (vmodlNs in self._verNameMap),
-        #  'Re-definition: {}'.format(vmodlNs)
-
         wireId = GetVersionNamespace(version)
         wireNs = wireId.split('/')[0]
-        self._verNameMap[vmodlNs] = version
-        self._verNameMapW[wireNs] = version
-        self._wireIdMap[vmodlNs] = wireId
-        self._wireIdMapW[wireNs] = wireId
+
+        # We use the string representation of the instance as unique name
+        if not _IsNameFrozen(wireNs, str(self)):
+            self._verNameMap[vmodlNs] = version
+            self._verNameMapW[wireNs] = version
+            self._wireIdMap[vmodlNs] = wireId
+            self._wireIdMapW[wireNs] = wireId
+            _TryFreezeName(wireNs, str(self))
+
+
         return wireId, wireNs
 
     def GetName(self, vmodlNs):
@@ -1610,15 +1663,13 @@ _VersionInit()
 
 newestVersions = _MaturitySet()
 ltsVersions = _MaturitySet()
-dottedVersions = _MaturitySet()
 oldestVersions = _MaturitySet()
 
-# Alias for backward compatibility. Required until VMC M11 is phased out.
-publicVersions = ltsVersions
-
 # AddVersion is not used in this module however it is imported from it in the
 # generated bindings so removing it will break the world.
-from .Version import AddVersion, IsChildVersion  # noqa: E402, F401
+from .Version import AddVersion, IsChildVersion, kind  # noqa: E402, F401
+
+_allowCapitalizedNames = kind != "OSS"
 
 byte = type("byte", (int, ), {})
 short = type("short", (int, ), {})
@@ -1626,15 +1677,15 @@ double = type("double", (float, ), {})
 if PY3:
     long = type("long", (int, ), {})
 URI = type("URI", (str, ), {})
-if not PY3 and _binaryIsBytearray:
-    # six defines binary_type in python2 as a string; this means the
+if not PY3 and kind == "OSS":
+    # binary_type in defined as a string, meaning that the
     # JSON encoder sees checksum properties as strings and attempts
     # to perform utf-8 decoding on them because they contain high-bit
     # characters.
     binary = type("binary", (bytearray,), {})
 else:
     binary = type("binary", (binary_type,), {})
-PropertyPath = type("PropertyPath", (six.text_type, ), {})
+PropertyPath = type("PropertyPath", (text_type, ), {})
 
 # _wsdlTypeMapNSs store namespaces added to _wsdlTypeMap in _SetWsdlType
 _wsdlTypeMapNSs = set()
@@ -1691,8 +1742,8 @@ del name, typ
 
 # unicode is mapped to wsdl name 'string' (Cannot put in wsdlTypeMap or name
 # collision with non-unicode string)
-_wsdlNameMap[six.text_type] = (XMLNS_XSD, 'string')
-_wsdlNameMap[CreateArrayType(six.text_type)] = (XMLNS_VMODL_BASE,
+_wsdlNameMap[text_type] = (XMLNS_XSD, 'string')
+_wsdlNameMap[CreateArrayType(text_type)] = (XMLNS_VMODL_BASE,
                                                 'ArrayOfString')
 
 # _wsdlMethodNSs store namespaces added to _wsdlMethodMap in _SetWsdlMethod
@@ -1738,7 +1789,7 @@ vmodlNames = {}
 
 
 # Add array type into special names
-for name, typ in six.iteritems(vmodlTypes.copy()):
+for name, typ in iteritems(vmodlTypes.copy()):
     if typ is not NoneType:
         try:
             arrayType = typ.Array
@@ -1787,6 +1838,36 @@ def GetVmodlType(name):
         raise KeyError(name)
 
 
+# List the VMODL types in the specified definitions
+#
+# @param defs Specifies the definitions from which to extract the names
+# @return a new list of the VMODL names in the specified dictionary
+def _ListVmodlTypes(defs):
+    with _lazyLock:
+        return list(defs.keys())
+
+
+# List all VMODL data types
+#
+# @return a new list of the VMODL names of all data types
+def ListDataTypes():
+    return _ListVmodlTypes(_dataDefMap)
+
+
+# List all VMODL managed types
+#
+# @return a new list of the VMODL names of all managed types
+def ListManagedTypes():
+    return _ListVmodlTypes(_managedDefMap)
+
+
+# List all VMODL enum types
+#
+# @return a new list of the VMODL names of all enum types
+def ListEnumTypes():
+    return _ListVmodlTypes(_enumDefMap)
+
+
 # Get VMODL type name from type
 #
 # @param typ vmodl type
@@ -1878,7 +1959,7 @@ class StringDict(dict):
 
     # Same as dict setdefault, except this will call through our __setitem__
     def update(self, *args, **kwargs):
-        for k, v in six.iteritems(dict(*args, **kwargs)):
+        for k, v in iteritems(dict(*args, **kwargs)):
             self[k] = v
 
     # Same as dict setdefault, except this will call through our __setitem__
@@ -1891,7 +1972,7 @@ class StringDict(dict):
 
     def __setitem__(self, key, val):
         """x.__setitem__(i, y) <==> x[i]=y, where y must be a string"""
-        if not isinstance(val, string_types):
+        if not isinstance(val, str_type):
             raise TypeError("key %s has non-string value %s of %s" %
                             (key, val, type(val)))
         return dict.__setitem__(self, key, val)
@@ -2074,3 +2155,10 @@ def AddBreakingChangesInfo(branchName, vmodlNamespace, count):
 
 def GetBreakingChanges():
     return _breakingChanges
+
+class VmomiJSONEncoder:
+    def __init__(self, *args, **kwargs):
+        raise Exception("No longer supported. Use pyVmomi.VmomiJSONEncoder instead.")
+
+def templateOf(typestr):
+    raise Exception("No longer supported. Use pyVmomi.VmomiJSONEncoder.templateOf instead.")
